
# Forum Python demonstration script
# Rotary device remote control via CMA180
# Dieter Bues, R&S Munich, Dep. 1MAA
# file: c:\W\FORUM\db_tt_22.py
# update: Oct. 6th, 2016


import visa
rm = visa.ResourceManager()

import time
import os

import numpy
import math

graphics=False  # show graphics or not
#graphics=True  # show graphics or not
loop_count=50


# open rotary device interface (=Arduino board) :
ardu = rm.open_resource('ASRL5::INSTR')

print('rotary device setup and selftest')
print(ardu.query("h"))
ardu.write("e1")

for i in range(0,10):
    ardu.write("l1")
    time.sleep(0.2)
    
for i in range(0,10):
    ardu.write("r1")
    time.sleep(0.2)    

ardu.write("d50")     # set step delay

print('ready !')
while(False):
    h=0




if graphics:
    fig = flib.pltframe.figure
    fig.clear()

# count the number of detected slopes
# the results decides about the button pressed
# values for CMA FFT frequency span of 20 kHz and FFT length 2k:
#     slopes detected:  ~175 <---> no button is pressed
#     slopes detected:  ~15  <--->  left button is pressed  
#     slopes detected:  ~3   <--->  right button is pressed
#     slopes detected:  >300 <--->  transmitter off or sigal too weak
def count_slopes(in_list):
    xcount=int(0)
    cur_res=in_list[0]>0.5  # current result
    
    for i in range(1,len(in_list)-1):
            if cur_res != (in_list[i]>0.5):
                cur_res = (in_list[i]>0.5)  # memorize current result
                xcount+=1
    return[xcount]


os.chdir('c:\\temp3')


idx=CMA.ask("*IDN?")
print(idx)

while (loop_count!=0):


    if graphics:
        fig.clear()

    # perform  measurement:
    CMA.write('CONFigure:AFRF:MEAS:MEV:REPetition SING')
    CMA.write('CONFigure:GPRF:MEAS:RFSettings:ENPower -35') # Exp. Pow., Minimum:  xx dBm at RFIN
    CMA.write('INITiate:GPRF:MEAS:FFTSanalyzer')
    
    xi=[]  # define empty list
    xq=[]
    xi=CMA.ask("FETCh:GPRF:MEAS:FFTSanalyzer:I?");  # get I-values from CMA
    xq=CMA.ask("FETCh:GPRF:MEAS:FFTSanalyzer:Q?");  # get Q-values

    xx=[]  # define empty list
    xfi=[]
    xfq=[]

    i=int()

    # convert to float   
    for i in range(0,len(xi)-1):
        xfi.append(float(xi[i]))
        xfq.append(float(xq[i]))   
 
    max_xi=max(xfi)
    max_xq=max(xfq)
    #take the larger one:
    if max_xi>max_xq:
        max_xx=max_xi
    else:
        max_xx=max_xq
       
    # normalize to one: 
    for i in range(0,len(xi)-1):
        xfi[i]/=max_xx
        xfq[i]/=max_xx    

    # calculate the vector magnitude
    for i in range(0,len(xi)-1):
        xx.append(math.sqrt(xfi[i]**2+xfq[i]**2))

    if graphics:
        ax = fig.add_subplot(111, autoscale_on = True)
        ax.patch.set_facecolor('white' )

        ax.grid(color = 'grey' , linestyle = '-')

        xxr=range(0,len(xx))


        ## Create plot

        ax.plot(xxr, xx, lw = 2, color = 'blue' )

        ## Show plot
        flib.pltframe.Show()
        

    loop_count-=1
    print('loop nr ',loop_count)
    
    cs=count_slopes(xx)[0] # counted pulses
    cs=count_slopes(xx)[0] # counted pulses
    print('counted slopes: ',cs)
   
    if cs < 10:
        print('right button')
        ardu.write("r5")
    elif cs < 20:
        print('left button')
        ardu.write("l5")
    elif cs < 200: 
        print('open')  
    else:
        print('no signal')   
    
